@@ -232,13 +232,20 @@ void gconf_load_state(appdata_t *appdata) {
 	else
 	  *gpx = gpx_parse(dialog, fname);
 
-	free(fname);
+	if(!*gpx) {
+	  /* restoring the gpx file failed, mark it as unusable, but save */
+	  /* its presence for later use */
+	  
+	  /* create fake entry to remember this file for next load attempt */
+	  *gpx = g_new0(gpx_t, 1);
+	  (*gpx)->filename = fname;
+	  (*gpx)->failed = TRUE;
+	} else
+	  free(fname);
       }
     }
 
-    /* use next gpx entry of this was loaded successfully */
-    if(*gpx)
-      gpx = &((*gpx)->next);
+    gpx = &((*gpx)->next);
   }
 
   gpx_busy_dialog_destroy(dialog);
