@@ -436,18 +436,17 @@ static gboolean goto_update(gpointer data) {
     gtk_label_set_text(GTK_LABEL(context->gotoc.bearing_label), "-----");
   }
 
-  float epe = gps_get_epe(context->appdata);
-  if(isnan(epe))
-    gtk_label_set_text(GTK_LABEL(context->gotoc.epe_label), "-----");
+  float eph = gps_get_eph(context->appdata);
+  if(isnan(eph))
+    gtk_label_set_text(GTK_LABEL(context->gotoc.eph_label), "-----");
   else {
     char str[16];
-    if(context->appdata->imperial) {
-      epe *= 3.2808;
-      snprintf(str, sizeof(str), "%.1f ft", epe);
-    } else
-      snprintf(str, sizeof(str), "%.1f m", epe);
+    /* distance needs to be given in km */
+    if(context->appdata->imperial)
+      eph /= (6371.0/3959.0);  // km to miles
 
-    gtk_label_set_text(GTK_LABEL(context->gotoc.epe_label), str);
+    distance_str(str, sizeof(str), eph/1000.0, context->appdata->imperial);
+    gtk_label_set_text(GTK_LABEL(context->gotoc.eph_label), str);
   }
 
   return TRUE;   // fire again
@@ -687,7 +686,7 @@ GtkWidget *goto_cache(cache_context_t *context) {
   gtk_table_attach_defaults(GTK_TABLE(table), 
 	    gtk_label_new(_("Est. error:")), 0,1,6,7);
   gtk_table_attach_defaults(GTK_TABLE(table),
-	    (context->gotoc.epe_label = gtk_label_new("-----")), 1,2,6,7);
+	    (context->gotoc.eph_label = gtk_label_new("-----")), 1,2,6,7);
 
   gtk_table_set_row_spacing(GTK_TABLE(table), 6, 16);
 
